local CrashSkinCheck = CrashAddon.CrashSkinCheck
local StateSetter = CrashAddon.StateSetter
local SpeedCalc = CrashAddon.SpeedCalc
local ActionButton = CrashAddon.ActionButton
local FailVibeCheck = CrashAddon.FailVibeCheck
local CootOnGround = CrashAddon.CootOnGround
local IsFalling = CrashAddon.IsFalling

//local variable to determine how much speed to add when the slide activates
local add_slide_speed = 10

local function CrawlAndSlide(player)
	if not CrashSkinCheck(player)
		return
	end
	
	local crashban = player.crashban
	//this resets everything and prevents moves if crash fails the vibe check
	if FailVibeCheck(player)
		player.normalspeed = skins[player.mo.skin].normalspeed
		player.height = skins[player.mo.skin].height
		crashban.slidetimer = 0
		return
	end
	
	//this resets the height and speed changes made by crawling and sliding
	if not (player.cmd.buttons & ActionButton(player)) and 
			crashban.slidetimer <= 0 and not crashban.bodyslam and 
			crashban.faceplant <= 0 and crashban.actionpress
		player.height = skins[player.mo.skin].height
		if player.normalspeed < skins[player.mo.skin].normalspeed
			player.normalspeed = skins[player.mo.skin].normalspeed
		end
		crashban.actionpress = false
	end
	//this handles what happens when crash hits a spring while crawling/sliding
	if CootOnGround(player.mo) and (player.cmd.buttons & ActionButton(player) or 
			crashban.slidetimer > 0) and player.mo.eflags & MFE_SPRUNG
		player.mo.state = S_PLAY_ROLL
		player.pflags = $ | PF_SPINNING
		crashban.slidetimer = 0
	end
	//this makes Crash crawl or start sliding
	if player.cmd.buttons & ActionButton(player) and CootOnGround(player.mo) and 
			crashban.spintimer <= 0 and not (player.pflags & PF_SPINNING)
		player.height = player.spinheight
		if not crashban.actionpress
			if SpeedCalc(player) > 3 * player.mo.scale and 
					crashban.slidetimer <= 0
				local slidespeed
				if SpeedCalc(player) > 30 * player.mo.scale
					slidespeed = SpeedCalc(player) + add_slide_speed * 
							player.mo.scale
				else
					slidespeed = (30 + add_slide_speed) * player.mo.scale
				end
				//this is done to nerf the slide force underwater
				if player.mo.eflags & MFE_UNDERWATER or player.mo.eflags & 
						MFE_GOOWATER or P_InSpaceSector(player.mo)
					slidespeed = $ - ($ / 3)
				end
				P_InstaThrust(player.mo, player.drawangle, slidespeed)
				S_StartSound(player.mo, sfx_cshsld)
				crashban.slidetimer = TICRATE / 2
			end
			crashban.actionpress = true
		elseif player.mo.state != S_PLAY_GLIDE and crashban.slidetimer <= 0 and
				crashban.faceplant <= 0
			player.normalspeed = 5 * FRACUNIT
			if player.mo.momx == player.cmomx and player.mo.momy == 
					player.cmomy
				StateSetter(player.mo, S_PLAY_CLING)
			else
				StateSetter(player.mo, S_PLAY_CLIMB)
			end
		end
	else
		if player.mo.state == S_PLAY_CLING
			player.mo.state = S_PLAY_STND
		elseif player.mo.state == S_PLAY_CLIMB
			player.mo.state = S_PLAY_WALK
		end
	end
	//this handles what happens during the slide and after
	if crashban.slidetimer > 0
		player.pflags = $ | PF_STASIS
		if player.mo.state != S_PLAY_GLIDE
			player.powers[pw_strong] = STR_ANIM|STR_WALL|STR_PUNCH
			player.mo.state = S_PLAY_GLIDE
		end
		P_SpawnSkidDust(player, 10 * player.mo.scale)
		player.normalspeed = 5 * FRACUNIT
		if not CootOnGround(player.mo) and IsFalling(player.mo)
			player.mo.momz = 0
		end
		crashban.slidetimer = $ - 1
	elseif player.mo.state == S_PLAY_GLIDE
		player.mo.state = S_PLAY_WALK
	end
end

//hooks
addHook("PlayerThink", CrawlAndSlide)